/*============================================================================
  File:     Heuristics.sql

  Summary:  There are numerous cases where statistics cannot be used - so 
            then what? Heuristics (fixed calculations that are rules-based)
            are used...
  
  SQL Server Version: SQL Server 2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- RESTORE CREDIT!!!

USE Credit
go

SET STATISTICS PROFILE OFF
go

-- Here - they yield one because column unique
-- Using a variable
DECLARE @test   int
SELECT @test = 9999
SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no = @test 
go

-- Here - we have an index but no "sniffing"
-- because the variable is obfuscating the value

-- use density_vector for "all density" (aka unknown)
-- simply put this is the average

UPDATE Member
	SET lastname = 'Tripp' 
WHERE member_no = 1234

-- with an index on LN
sp_helpindex member
--drop index member.LName
CREATE INDEX LNInd on member(LastName)
--UPDATE STATISTICS member WITH FULLSCAN
go
DECLARE @test   varchar(15)
SELECT @test = 'Tripp'

-- Here it's obfuscated
SELECT m.* 
FROM dbo.member AS m 
WHERE m.lastname = @test -- UNKNOWN

-- Here it's a literal (and, we have an index)
SELECT m.* 
FROM dbo.member AS m 
WHERE m.lastname = 'Tripp'
go

-- To see where the stats come from, use: 
DBCC SHOW_STATISTICS ('member', 'LNind')
go


-- What if we didn't have those stats? And,
-- we don't allow SQL Server to create any...

-- Drop the index on LN AND w/o auto create stats
DROP INDEX member.LNInd
go

ALTER DATABASE [Credit] SET AUTO_CREATE_STATISTICS OFF
go

-- These are only needed if you created/ran other queries
-- DROP STATISTICS member.[_WA_Sys_00000002_0CBAE877]
-- DBCC SHOW_STATISTICS (member, '_WA_Sys_00000002_0CBAE877')
-- Quickly check, are there any column stats?
exec sp_helpindex member
go
exec sp_helpstats member
go

-- What happens?
DECLARE @test   varchar(15)
SELECT @test = 'Tripp'
SELECT m.* 
FROM dbo.member AS m 
WHERE m.lastname = @test -- No stats... heuristics!
go

SELECT m.* 
FROM dbo.member AS m 
WHERE m.lastname = 'Tripp' -- No stats... heuristics!
go

-- These are all heuritics...

-- Here - they yield 30% *no matter what*
DECLARE @test   int
SELECT @test = 9999
SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no > @test -- obfuscation... therefore heuristics (not equality)
go

DECLARE @test   int
SELECT @test = 2
SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no > @test -- obfuscation... therefore heuristics (not equality)
go

-- Again - they yield 30% 
DECLARE @test   int
SELECT @test = 1
SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no < @test
go

-- Here - they yield 9% *no matter what*
DECLARE @lowerbound   int
DECLARE @upperbound   int
SELECT @lowerbound = 1
SELECT @upperbound = 10000

SELECT m.* 
FROM dbo.member AS m 
WHERE m.member_no BETWEEN @lowerbound AND @upperbound
go

-- What about a comparison between columns?
DROP TABLE member2
go

SELECT member_no as MNO1,
        member_no AS MNO2,
        lastname,
        firstname,
        middleinitial 
INTO member2
FROM member
go

SELECT m.* 
FROM dbo.member2 AS m
WHERE m.MNO1 = m.MNO2
-- Here it looks like the estimate is 10%...