-- Talking about non-seekable expressions
SELECT ... WHERE msal * 12 > 100000		-- the column is not isolated (non-seekable)
SELECT ... WHERE msal > 100000/12		-- the column IS isolated (seekable)
SELECT ... WHERE 100000/12 < msal		-- left/right is irrelevant, still seekable

-- What about variables and parameters??

declare @ansal	decimal(10, 2) = 100000
SELECT * FROM tname WHERE msal * 12 < @ansal
	-- always stinks (non-seekable expression)

SELECT * FROM tname WHERE msal < @ansal/12
	-- cannot be sniffed 
	-- the VALUE of the *VARIABLE* is unknown during optimization
	-- as a result, we may not get an optimal plan

SELECT * FROM tname WHERE msal < 100000/12
	-- can be sniffed, most optimal
	-- the VALUE of the comparison is a literal (completely known/calculated ONCE)
	-- as a result, SQL Server can use statistics and "sniff" the parameter
	-- to determine a plan for that particular value (based on the number of rows that match the expression)
go

CREATE PROC p1
	(@ansal decimal(10, 2))
AS
SELECT * FROM tname WHERE msal < @ansal/12 
	-- param sniffing
go

CREATE PROC p2
	(@ansal decimal(10, 2))
AS
DECLARE @msal decimal(10, 2)
SELECT @msal = @ansal / 12
SELECT * FROM tname WHERE msal < @msal
	-- unknown at compilation
	-- cannot sniff. Will use density vector



